 /******************************************************************************
 * This demo for the MAXQ2000 uses an LCD and a thermistor or
 * 1-Wire thermometer to display the current temperature and
 * drive a fan via PWM output.
 *
 * Features demostrated in this demo:
 *   1) LCD controller
 *   2) Timers
 *   3) PWM control
 *   4) SPI communication
 *   5) Analog to digital conversion with external hardware (MAX1407)
 *   6) 1-wire for temperature
 *
 * Functional overview:
 *   The timer is set to generate an interrupt at a regular interval.  On an
 *   interrupt, the part performs a temperature reading, updates the LCD, and
 *   outputs the current temperature to the serial port.  The units will also
 *   toggle between Celcius and Fahrenheit after 20 interrupts. In addition 
 *   there is a PWM whose output is driven by the temperature; the higher the
 *   the temperature the faster the motor runs. 
 *
 *   For simplicity, the temperature reading in this demo application is not 
 *   entirely accurate.  It is reasonably accurate at and around room 
 *   temperature. The temperature can be derived from the thermistor and the 1407
 *   or an iButton (1920) via 1-wire.
 *
 * Parts required:
 *    (1) MAXQ2000 EV-KIT Rev B with attached LCD (+ JTAG board, power, serial cable)
 *    (1) LED or fan for attaching to the PWM and any neeeded resistors  and/or power supplies
 * Either:
 *    (1) 10K Resistor
 *    (1) RL1005-5744-103-SA Thermistor 
 *       [http://rocky.digikey.com/scripts/ProductInfo.dll?Site=US&V=45&M=RL1005-5744-103-SA]
 *    OR
 *    (1) iButton (1920)
 *
 * Wiring requirements (for the Thermistor):
 *
 *  The thermistor should be wired as follows:
 *
 *    10K Ohm between pin 1 of J7 and GND (1407:IN0 & GND)
 *    Thermistor between pin 1 and pin 7 of J7 (1407:IN0 & 1407:OUT2)
 *    Short between pin 7 and 8 of J7 (1407:OUT2 & 1407:FB2)
 *
 *  These switches should be turned on:
 *
 *    SW1.7               : Enable Serial 0 transmit
 *    SW6.2, SW6.5        : Enable pushbuttons
 *    SW3.1 to SW3.8      : All on to enable comm with the MAX1407 (only needed if using thermistor)
 *
 *  These switches should be turned off:
 *     SW1.3,SW1.4 (P5.2 and P7.1)
 *
 *  Once everything has been wired, you can run the demo from within IAR's
 *  Embedded Workbench IDE.  After the demo is running, you should see the
 *  current temperature displayed on the LCD in degrees Celcius.
 *
 *****************************************************************************/
 
#include <intrinsics.h>
#include "iomaxq200x.h"

#define PWMFREQ	100000        // Change this for desired PWM frequency
#define CPUFREQ	16000000      // Change this to match current clock frequency
#define MIN_TICKS 0           // Minimum number of timer ticks to enable fan to turn
#define POLLING_INTERVAL 500  // Number of milliseconds between temperature checks

// Initial temperature values
#define INIT_BASE_TEMP 75
#define INIT_MAX_TEMP 80

#include "ownet.h"
#include "temp10.h"

/*
 * Command codes for the MAX1407
 */
#define WR_ADC       0x080                    // 10   00000    0
#define WR_MUX       0x082                    // 10   00001    0
#define WR_DAC2      0x08A                    // 10   00101    0
#define WR_RUN       0x0BA                    // 10   11101    0

#define RD_ADC       0x0C0                    // 11   00000    0
#define RD_DATA      0x0C4                    // 11   00010    0
//#define RD_DAC2      0x0CA                    // 11   00101    0
#define RD_STAT      0x0CC                    // 11   00110    0

/* 
 * Some definitions for LCD patterns.  For a full LCD digit, just index
 * into the PATTERNS array and write that value to the appropriate LCD register.
 *
 * Compatible with the MAXQ2000 EVKit Rev. B
 */
#define LCD0_PATTERN_C    0x039
#define LCD0_PATTERN_F    0x071

#define LCD_PATTERN_0     0x03F
#define LCD_PATTERN_1     0x006
#define LCD_PATTERN_2     0x05B
#define LCD_PATTERN_3     0x04F
#define LCD_PATTERN_4     0x066
#define LCD_PATTERN_5     0x06D
#define LCD_PATTERN_6     0x07D
#define LCD_PATTERN_7     0x007
#define LCD_PATTERN_8     0x07F
#define LCD_PATTERN_9     0x067

int PATTERNS[] = { LCD_PATTERN_0, LCD_PATTERN_1, LCD_PATTERN_2, LCD_PATTERN_3, LCD_PATTERN_4, 
                   LCD_PATTERN_5, LCD_PATTERN_6, LCD_PATTERN_7, LCD_PATTERN_8, LCD_PATTERN_9 };

/*
 * Flag - are we displaying celsius or fahrenheit temperatures?
 */
unsigned char celsius = 0;

/*
 * Counts how many temperature readings we have made.  After 20, we will change
 * our display mode (celsius <--> fahrenheit)
 */
int count = 0;

/*
 * Debug variable(s)
 */
int last_state = 0;  				// States whether to increase (0) base_temp or (1) max_temp 

/*
 * Global variables for thresholds and changed via pushbuttons
 */
int base_temp = INIT_BASE_TEMP;				// Sets lower fan threshold
int max_temp = INIT_MAX_TEMP;				// sets upper fan threshold
// Multiplier used to determine compare value per degree
int xplier = ( (CPUFREQ / PWMFREQ) / (INIT_MAX_TEMP - INIT_BASE_TEMP) ); 
              
/*******************************************************************
 * Simple calculation funtion which converts Celcius to Fahrenheit
 */
int CtoF(int temp)
{
  return (int)(((double)temp) * 9.0 / 5.0 + 32.0);
}

/******************************************************************************
 * Performs an SPI transaction...output the value of 'spib' onto the
 * SPI bus, and return whatever was read.
 */
unsigned int sendSPI(unsigned int spib)
{
  SPIB = spib;                  // Load the data to send
  while(SPICN_bit.STBY);        // Loop until the data has been sent.
  SPICN_bit.SPIC = 0;           // Clear the SPI tansfer complete flag.
  return SPIB;  
}

/******************************************************************************
 * Gets an A->D reading from the MAX1407.  Communicates over SPI.
 * 16-bits are returned from the 1407.
 */
unsigned int getADCReading()
{
  unsigned int spiData;
  
  sendSPI(RD_ADC);                    // Read the 1407's ADC register.
  spiData = sendSPI(0x0FF);
  spiData |= 0x01;                    // Set the start conversion bit.
  sendSPI(WR_ADC);                    // Write the new value back into
  sendSPI(spiData);                   //   the 1407 register.
  
  do{
    sendSPI(RD_STAT);                 // Read the 1407's Status register.
    spiData = sendSPI(0x0FF);
  } while((spiData & 0x02) == 0x00);  // Bit 1 indicates AtoD conversion complete.
  
  sendSPI(RD_DATA);                   // Send the command to read the 1407's Data register.
  SPICF = 0x04;                       // Set up SPI to 16-bit mode.
  spiData = sendSPI(0x0FFFF);         // Read the Data register.
  SPICF = 0x00;                       // Put SPI  back into 8-bit mode.
  
  return spiData;
}

/******************************************************************************
 * Initializes the MAX1407.  Configures SPI for rising edge, 8-bit mode, master
 * mode.  3 readings must be taken before the 1407's results become valid.
 */
void init1407()
{
  unsigned int spiData;
  int i;
  
  PD5 |= 0x070;             // Set SPI output pins (CS, SCLK, DOUT) as output.
  PD5 &= ~0x080;            // Set SPI input pin (DIN) as input.

  SPICK = 0x10;             // Configure SPI for rising edge, sample input
  SPICF = 0x00;             //   on inactive edge, 8 bit, divide by 16.
  SPICN_bit.MSTM = 1;       // Set Q2000 as the master.
  SPICN_bit.SPIEN = 1;      // Enable SPI.

  PO5 &= ~0x10;             // Set /CS low.

  sendSPI(WR_RUN);          // Make sure 1407 is up and running by putting
                            //   it into RUN mode.

  sendSPI(WR_ADC);          // Set up the 1407 ADC to sample at 30Hz, unbuffered,
  sendSPI(0x10);            //   in unipolar mode.  Also sets the PGA to 1.

  sendSPI(WR_MUX);          // Set up the 1407 MUXs.  The positive MUX is set to
  sendSPI(0x2C);            //   REF and the negative MUX is set to IN0.

  sendSPI(RD_ADC);          // Read the 1407's ADC register.
  spiData = sendSPI(0x0FF);
  spiData |= 0x81;          // Set the calibration & start conversion bit.
  sendSPI(WR_ADC);          // Write the new value back into
  sendSPI(spiData);         //   the 1407 register.
  do {
    sendSPI(RD_STAT);                 // Read the 1407's Status register.
    spiData = sendSPI(0x0FF);
  } while((spiData & 0x02) == 0x00);  // Bit 1 indicates AtoD conversion complete.

  sendSPI(WR_DAC2);                   // Set up the DAC2 to output 1.25V.  (FB2
                                      //   must be directly connected to OUT2).
  SPICF_bit.CHR = 1;                  // Set SPI to 16-bit mode.
  sendSPI(0x0FFC0);

  SPICF_bit.CHR = 0;                  // Put SPI back into 8-bit mode.

  for(i = 0; i < 3; i++)              // You must do three ADC readings on the 1407
  {                                   // before the results become valid.
    getADCReading();
  }
}
/******************************************************************************
 * Converts an A->D reading received from the MAX1407 into a celsius 
 * temperature approximation.
 */
float convertToTemp(unsigned int adc)
{
  double temp = -0.00135477 * (double)adc + 69.17;
  return (float)temp;
}



/******************************************************************************
 * Lights up the solitary 'one' segment on our LCD panel.
 */
show100s()
{
  LCD4 = 0x40;
}

/******************************************************************************
 * Returns the value that will need to be placed in one of LCD0-LCD3 to display 
 * a digit 0-9.  No bounds checking is done here.  If you ask for a digit other
 * than 0-9, you will get a bogus display.
 */
int getLCDDigit(int digit)
{
    return PATTERNS[digit];
}

/******************************************************************************
 * Change duty cycle of PWM by modifying compare value and 
 * boundaries as needed. Default Start values are:
 * T2V0 = 0xFFFF-(CPUFREQ / PWMFREQ); 
 * T2R0 = T2V0
 * T2C0 = T2R0+1
 * Output of the PWM changes whenever timer crosses the compare value.
 * The PWM starts out off, so when the timer reaches the compare value
 * it turns on. When the timer reaches FFFF(hex) the PWM output
 * switches again, and turns it off.
 */
void drivePWM(int temp2)
{
  int temp = CtoF(temp2);			// Converts temperature from Celcius to Fahrenheit

  T2CNA0_bit.TR2 = 0;			// Turn timer off
  T2CNA0_bit.T2OE0 = 0;			// Turns PWM output off
  T2CNA0_bit.T2POL0 = 0;           	// Changes polarity of PWM so that it starts out off
  T2CNA0_bit.T2OE0 = 1;			// Turns PWM output on
  T2V0 = T2R0;				// Manually reload timer value
	
  if(temp <= base_temp) 			// Below this temperature the fan is off
  {
      // Force port pin P6.5 low
      PD6 |= 0x20;
      PO6 &= 0xDF;
  }
  else
  {
    if( temp >= max_temp )		// Beyond this temperature the fan is full speed
    {
      // Force port pin P6.5 high
      PD6 |= 0x20;
      PO6 |= 0x20;
    }
    else				// Set the fan speed according to temperature
    {                                                                                 
      T2C0 = 65535 - ((temp - base_temp) * xplier);
      if( (65535 - T2C0) < MIN_TICKS)
        T2C0 = 65535 - MIN_TICKS;
      T2CNA0_bit.TR2 = 1;	            // Restart timer
    }
  }
}

/******************************************************************************
 * Take a celsius temperature and output it to the LCD drive.  Figure out if
 * we want to display a Celsius or Fahrenheit value also.  Change the output
 * units appropriately.
 */
void showTemp(int temp)
{
  LCFG_bit.DPE = 0;   // disable display.
  // Clear the display of everything but the temperature units (F or C).
  LCD0 = ( (celsius == 0) ? LCD0_PATTERN_F : LCD0_PATTERN_C);

  // clear the digits 
  LCD1 = LCD2 = LCD3 = LCD4 = 0;
  if(last_state == 0) 			// Display correct threshold being edited
    LCD1 = 0x08;
  else 
    LCD1 = 0x01;
    
  if(celsius == 0)
  {
    // Convert to Fahrenheit.
    temp = CtoF(temp); 			// Convert to Fahrenheit for calculations
  }

  if (temp > 199)			// If temperature is 200+
  {
      temp = temp % 200;
  }

  if(temp > 99)			// If the temperature is 100+
  {
    show100s();                         // Show a '1' on the LCD.
    temp -= 100;                        // Adjust the temperature variable.
  }

  LCD3 = getLCDDigit(temp / 10);        // show the 10's place on the LCD
  LCD2 = getLCDDigit(temp % 10);        // show the 1's place on the LCD
  LCFG_bit.DPE = 1;   // Enable display.

}

int readiButtonTemp(float *temp)
{
  uchar SerialNum_buf[8];                         // Holds serial number of iButton
  owFamilySearchSetup(0,0x10);                    // Setup the search to look for an iButton Thermometer
  if(owFirst(0,TRUE,FALSE)==TRUE)                 // Check to see if the device can be found
  {       
    owSerialNum(0,SerialNum_buf,1);
    if(SerialNum_buf[0]==0x10)	                  // Make sure the iButton is the DS1920/DS1820
    {
      return ReadTemperature(0,SerialNum_buf,temp);  // Check to make sure the temperature is read correctly
    }
  }
  
  return FALSE;
}

/******************************************************************************
 * Get an A->D reading, convert it to a temperature, and display it.  This 
 * function is called under the timer interrupt.
 */
void readTemp()
{
  float temp;				// Holds temperature
  int adc;

  if(readiButtonTemp(&temp)==FALSE)			 // Check to see if the device can be found
  {
    // If there is no iButton device found
    adc = getADCReading();            // Read the thermistor value.
    temp = convertToTemp(adc);        // Convert this value to degrees Celsius.  
  }
  
  showTemp(temp);                   // Display temperature
  drivePWM(temp);                   // Update fan speed
}

/******************************************************************************
 * Initialize Timer 1 (type 2).  With a 13.5 MHz crystal, we can get a 0.5 second 
 * interrupt period if the timer is run at clock/128...
 *
 *         timerticks * 128 = seconds * frequency
 *         timerticks = 0.5 * 13500000 / 128 ~= 52734
 *
 * Since our timers tick upwards, we want our reload value to be:
 *         reload = 64K - timerticks = 65536 - 52734 = 0x3203
 */
void initTimer1()
{
  T2V1 = 0xFFFF - (CPUFREQ/128/1000*POLLING_INTERVAL);  // Set reload value.
  T2R1 = T2V1;                            // Set current timer value.
  T2C1 = 0x00000;                         // Set compare value

  T2CFG1_bit.T2DIV = 7;                   // Set div 128 mode.

  T2CNA1_bit.TR2 = 1;                     // Start the timer.
  T2CNA1_bit.T2POL0 = 1;                  // Set polarity for T2OE0 to high.
  T2CNA1_bit.ET2 = 1;                     // Enable timer interrupts.

  T2CNB1_bit.T2OE1 = 1;                   // Enable timer output.

  IMR_bit.IM4 = 1; 			  // Enable the interrupts for module 4.
}
/******************************************************************************
 * Initialize Timer 0 (Type 2).
 */

void initTimer0()
{
  T2V0 = 0xFFFF - (CPUFREQ / PWMFREQ);	        // Set current timer value
  T2R0 = T2V0;                                  // Set reload value
  T2C0 = T2R0+1;                                // Set compare value to reload value +1

  T2CNA0_bit.T2OE0 = 0;	                        // Turns PWM output off
  T2CNA0_bit.T2POL0 = 0;                        // Changes polarity of PWM so that it starts out "off"
  T2CNA0_bit.T2OE0 = 1;	                        // Turns PWM output on
  T2CFG0 = 0x00;                                // Set timer divide at 1
} 
/******************************************************************************
 * Setup the external interrupts for EI 10 and 15 (pins P5.2 and P7.1).
 * NOTE: Pin P5.2 can not be used because the timer is using the physical
 * resource of that pushbutton. The external interrupt 15 can be used if desired.
 */
void initPushButtons()
{
  EIE1 = 0x84;                          // enable interrupts 15 and 10
  EIES1 = 0x84;                         // set up edge transitions
  IMR_bit.IM1 = 1;                      // turn on interrupts for module 1 
}

/******************************************************************************
 * The interrupt handler for module 1, including my external interrupts.
 * Must be careful in here to make sure I react only to the interrupts I 
 * want, since some pins are dual purpose.
 */
#pragma vector = 1
__interrupt void pushButtonInterrupt()
{
  if (EIF1 & 0x04)  //interrupt 10                    // Change which threshold to changed
  {
    if(last_state == 0)                              // Modify max_temp instead of base_temp
    {
      LCD1 = 0x01;				// Change display to underscore
      last_state = 1;
    }
    else                                             // Modify base_temp instead of max_temp
    {
      LCD1 = 0x08;
      last_state = 0;				//Change display to overscore
    }
  }
  if (EIF1 & 0x80)	//interrupt 15                     // Increase threshold for base or max temps
  {
    if(last_state == 0)                                
    {                                                 // Increase base_temp
      if(base_temp < (max_temp -1))                   
      {
        ++base_temp;
        if(base_temp >99)                            // If over 99, print 1 for 100's place
          LCD4 = 0x40;
        else
          LCD4 = 0;                               // Else print nothing
        LCD3 = getLCDDigit( (base_temp /10) %10);		// Print 10's digit
        LCD2 = getLCDDigit(base_temp %10);           // Print 1's digit
      }
      else if (base_temp == max_temp -1)              // If base temp is going to equal max temp
      {                                               // set base temp to 50 instead of increasing
        base_temp = 50;
        LCD3 = getLCDDigit(5);                       // Print 50 to display
        LCD2 = getLCDDigit(0);
      }
    }
    else if (last_state == 1)                          // Increase max_temp
    {
      if(max_temp < 149)
      {
        ++max_temp;
        if(max_temp > 99)
          LCD4 = 0x40;										//show 100s digit
        else
          LCD4 = 0;
        LCD2 = getLCDDigit(max_temp %10);     	 	// Show 1s digit
        LCD3 = getLCDDigit( (max_temp / 10) %10);       // Show 10s digit
      }
      else
      {
        if (max_temp == 149)                         // If max_temp is at limit then cycle 
          max_temp = base_temp+1;                   // to be 1 degree over base temp
        if(max_temp > 99)
          LCD4 = 0x40;                              // Clear 100s digit
        else 
          LCD4 = 0;
        LCD2 = getLCDDigit(max_temp %10);			// Show 1s digit
        LCD3 = getLCDDigit( (max_temp / 10) %10); // Show 10s digit
      }
    }
    xplier = ( (CPUFREQ/PWMFREQ) / (max_temp - base_temp));    // Re-calculate multiplier
  }
  EIF1 = 0;                                            // Clear External Interrupt Flag
}

/******************************************************************************
 * The Interrupt handler for Timer 1.  Occurs whenever the timer overflows.
 */
#pragma vector = 4 
__interrupt void timer1Interrupt()
{
  T2CNA1_bit.TR2 = 0;              // Stop the timer.
  
  if(++count == 20) 					  // Change the units occasionally.
  {
    count = 0;
    celsius = ~celsius;
  }

  readTemp();           //Get and display the current temperature.
  T2CNB1_bit.TF2 = 0;   //Clear the overflow flag.
  T2CNB1_bit.TCC2 = 0;  //Clear the overflow flag.
  
  // output an 'alive' blip with the first decimal point, LCD0.7
  if (count & 1)
    LCD0 = LCD0 | 0x80;
  else
    LCD0 = LCD0 & 0x7F;
  T2CNA1_bit.TR2 = 1;                     // Start the timer.
}

/******************************************************************************
 * Initialize the LCD to run off the 32 kHz, with external drive power.
 */
void initLCD()
{
  //LCRA_bit.FRM = 7;   // Set up frame frequency.
  //LCRA_bit.LCCS = 1;  // Set clock source to HF/128.
  //LCRA_bit.DUTY = 0;  // Set up static duty cycle.
  //LCRA_bit.LRA = 0;   // Set R-adj to 0.
  //LCRA_bit.LRIGC = 1; // Select external LCD drive power.
  LCRA = 0x03E0;        // Do all configuration changes at once

  LCFG_bit.PCF = 0x0F;  // Set up all segments as outputs.
  LCFG_bit.OPM = 1;     // Set to normal operation mode.
  LCFG_bit.DPE = 1;     // Enable display.
  LCD1 = 0x08;
}
/******************************************************************************
 * Initialize the 1-Wire system.
 */ 
void initOW()
{
  owAcquire(0,NULL);		// Acquire 1-Wire Net
}

/******************************************************************************
 * Main application entry point.  Initialize the serial port, timer, SPI/1407,
 * 1-Wire, LCD, and external interrupts.  Then just do nothing.  The interrupts will
 * take care of everything else.
 */
 
void main()
{
  initTimer1();	            // Initialize timer1 for thermistor/iButton polling
  initTimer0();             // Initialize timer0 for PWM output
  init1407();               // Initialize the 1407
  initOW();                 // Initialize the 1-Wire subsystem
  initLCD();                // Initialize LCD display
  initPushButtons();        // Initialize pushbuttons

  __enable_interrupt();     // Enable global interrupts

  while(1);
}

